/****************************************************************
LevelCon.C
CS 481 - Team E
Spring 1998
Hummingbird Project - Dr. Frincke
Coded by: Jamie Marconi
Description: This is the implementation of the level container object.
             This object stores and organizes all the level objects after
             they are created from the contents of the configuration
             files.
*****************************************************************/
/*
 Copyright (C) 1997-1998 The HummingBird Project - hummer@cs.uidaho.edu

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "LevelCon.h"

static char LevelCon_C[] = "$Id: LevelCon.C,v 1.3 1998/03/06 23:00:10 marc9442 Exp $";

/**********
Function: LevelCon::LevelCon
Purpose:  Basic constructor. Initialize some values.
**********/

LevelCon::LevelCon()
{
  // List starts out empty
  Head = NULL;
  Last = NULL;
}


/**********
Function: LevelCon::~LevelCon
Purpose:  Destructor. Destroy the linked lists of trusts.
**********/

LevelCon::~LevelCon()
{
  Level *TempPointer;
  Level *DeletePointer;

  // Start at the top of the list
  TempPointer = Head;
  while(TempPointer != NULL){
    // Delete the top and move to the next
    DeletePointer = TempPointer;
    TempPointer = TempPointer->Next;
    delete DeletePointer;
  }
}


/**********
Function: LevelCon::AddLevel
Purpose:  This function adds a trust object to the container.
          Right now this function returns 1 all the time.
**********/

int LevelCon::AddLevel(Level *NewLevel)
{
  // Put the new level node at the top of the list if there is no list
  if(Head == NULL){
    Head = NewLevel;
  }
  // Put the new level node at the end of the list if there is a list
  else{
    Last->Next = NewLevel;
  }   
  // Fix up the end of the list
  Last = NewLevel;
  Last->Next = NULL;
  return 1; 
}


/**********
Function: LevelCon::SetDefaultTrust
Purpose:  This function sets the defualt trust level. This trust level is 
          assigned to any message from a host that is not found in the 
	  trust container.
**********/
  
void LevelCon::SetDefaultTrust(int Level)
{
  DefaultTrustLevel = Level;
}

/**********
Function: LevelCon::SetDefaultCoop
Purpose:  This function sets the defualt cooperation level. This cooperation
          level is assigned to any message from a host that is not found in 
          the cooperation container.
**********/
  
void LevelCon::SetDefaultCoop(int Level)
{
  DefaultCoopLevel = Level;
}

/**********
Function: LevelCon::SetDefaultInteg
Purpose:  This function sets the defualt integrity level. This integerity 
          level is assigned to any message from a host that is not found in
          the integerity container.
**********/
  
void LevelCon::SetDefaultInteg(int Level)
{
  DefaultIntegLevel = Level;
}


/**********
Function: LevelCon::GetTrust
Purpose:  This function takes an IP address as a parameter and returns the 
          trust associated with that address. Default is returned if the
	  host ip is not found.
**********/

int LevelCon::GetTrust(char *HostIP)
{
  Level *TempPointer;
  int Level;
  char *Host;

  // Allocate memory
  Host = new char[IP_ADDRESS_SIZE];

  // Temp pointer to use to hop through list
  TempPointer = Head;
  // start level with default
  Level = DefaultTrustLevel;
  // Look up numeric host address to compair
  ConvertIP(HostIP, Host);
  
  // for each host in the list
  while(TempPointer != NULL){
    // Check to see if its an explicit address
    if(!strcmp(TempPointer->EndHostIP,"0")){

      // Check to see if its the Host IP we are looking at
      if(!strcmp(TempPointer->StartHostIP, Host)){ 

	// We found it set level and break the loop
	Level = TempPointer->TrustLevel - '0';       
	break;
      }
    }
    else {
        
      // See if its in the range
      if( strcmp(TempPointer->StartHostIP, Host) <= 0 && 
	  strcmp(TempPointer->EndHostIP, Host) >= 0){
	
	// It is set the level and break the loop
	Level = TempPointer->TrustLevel - '0';
	break;
      }
      
    }
    // Try next node in the list
    TempPointer = TempPointer->Next;
  }

  // Free memory
  delete[] Host;
  
  return Level;
}

/**********
Function: LevelCon::GetCoop
Purpose:  This function takes an IP address as a parameter and returns the 
          Cooperation associated with that address. Default is returned if the
	  host ip is not found.
**********/

int LevelCon::GetCoop(char *HostIP)
{
  Level *TempPointer;
  int Level;
  char *Host;

  // Allocate memory
  Host = new char[IP_ADDRESS_SIZE];

  // Look up numeric host address to compair
  ConvertIP(HostIP, Host);

  // Temp pointer to use to hop through the list
  TempPointer = Head;
  // start level with the default
  Level = DefaultCoopLevel;

  // for each host in the list
  while(TempPointer != NULL){

    // Check to see if its an explicit address
    if(!strcmp(TempPointer->EndHostIP,"0")){
      
      // Check to see if its the host ip we are looking at
      if(!strcmp(TempPointer->StartHostIP, Host)){ 

	// We found it set level and break the loop
	Level = TempPointer->CoopLevel - '0';       
	break;
      }
    }
    else{
      // See if its in the range
      if( strcmp(TempPointer->StartHostIP, Host) <= 0 && 
	  strcmp(TempPointer->EndHostIP, Host) >= 0){
	
	// It is set the level and break the loop.
	Level = TempPointer->CoopLevel - '0';
	break;
      }
      
    }

    // Try the next node in the list
    TempPointer = TempPointer->Next;
  }

  // Free memory
  delete[] Host;
  
  return Level;
}


/**********
Function: LevelCon::GetInteg
Purpose:  This function takes an IP address as a parameter and returns the 
          integrity associated with that address. Default is returned if the
	  host ip is not found.
**********/

int LevelCon::GetInteg(char *HostIP)
{
  Level *TempPointer;
  int Level;
  char *Host;

  // Allocate memory
  Host = new char[IP_ADDRESS_SIZE];

  // look up numeric host address to compair
  ConvertIP(HostIP, Host);

  // Temp pointer to use to hop through list
  TempPointer = Head;
  // start levle with default
  Level = DefaultIntegLevel;

  // for each host in the list
  while(TempPointer != NULL){
    
    // Check to see if its an explicit address
    if(!strcmp(TempPointer->EndHostIP,"0")){
      
      // Check to see if its the Host we are looking at
      if(!strcmp(TempPointer->StartHostIP, Host)){ 
	
	// We found it set level and break the loop
	Level = TempPointer->IntegLevel - '0';       
	break;
      }
    }
    else{
      // See if its in the range
      if( strcmp(TempPointer->StartHostIP, Host) <= 0 && 
	  strcmp(TempPointer->EndHostIP, Host) >= 0){

	// It is set the level and break the loop
	Level = TempPointer->IntegLevel - '0';
	break;
      }
      
    }
    TempPointer = TempPointer->Next;
  }

  // Free memory
  delete[] Host;
  
  return Level;
}
