/****************************************************************
MessageFilter.h
CS 481 - Team A
Spring 1997
Hummingbird Project - Dr. Frincke
Coded by: Pete Neisen
Description: The message filter object. This is used to store a message
             filter when it is loaded from file and stored in the 
	     message filter container object.
*****************************************************************/
/*
 Copyright (C) 1997-1998 The HummingBird Project - hummer@cs.uidaho.edu

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef _MessageFilter_h
#define _MessageFilter_h

#include "Hummer.h"

// HostList is used as part of the message filter and other things.
class HostList{
 public:
  char StartHostIP[IP_ADDRESS_SIZE];  // Unique address, or start of range
  char EndHostIP[IP_ADDRESS_SIZE];    // End of range or NULL if unique
  HostList *Next;                     // Pointer to next node in list
};

class PeerList {
 public:
  char ModHostIP[IP_ADDRESS_SIZE];  // address of moderator
  char GroupName[NORMAL_STRING_SIZE]; // Name of Peer group on moderator
  PeerList *Next;
};

class MessageFilter
{
 public:
  char RegExp[NORMAL_STRING_SIZE];  // Regular expression matched to message
  char StartDate[DATE_SIZE];        // date to start filtering between
  char EndDate[DATE_SIZE];          // date to end filtering between
  char StartTime[TIME_SIZE];        // time to start filtering between
  char EndTime[TIME_SIZE];          // time to end filtering between
  char TrustLevel[LEVEL_SIZE];      // Trust level must be greater then
  char CoopLevel[LEVEL_SIZE];       // Cooperation Level must be greater than
  char IntegLevel[LEVEL_SIZE];      // Integrity level must be greater than
  HostList *FromHostList;           // List of hosts to accept messages from
  char Log;                    // Action log this message
  char Alert;                  // action Alert console interface
  HostList *ForwardHostList;   // List of hosts to forward message to
  PeerList *ForwardPeerList;   // List of peer groups to forward message to
  MessageFilter *Next;         // pointer to next node in filter list
  MessageFilter *NewNext;    // Used to thread the list in MatchExp()
};

#endif
