/****************************************************************
MessageFilterCon.C
CS 481 - Team A
Spring 1997
Hummingbird Project - Dr. Frincke
Coded by: Pete Neisen
Description: This file is the implementation of the message filter
             container object. This object is used to store and 
	     organize the message filters that the messages are compared
	     against.
*****************************************************************/
/*
 Copyright (C) 1997-1998 The HummingBird Project - hummer@cs.uidaho.edu

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

static char MessageFilterCon_C[] = "$Id: MessageFilterCon.C,v 1.6 1998/03/06 23:00:11 marc9442 Exp $";

#include "MessageFilterCon.h"


/**********
Function: MessageFilterCon::MessageFilterCon
Purpose:  Basic constructor. A few variables initialized.
**********/

MessageFilterCon::MessageFilterCon()
{
  // Start filter list out empty
  Head = NULL;
  Last = NULL;
}


/**********
Function: MessageFilterCon::~MessageFilterCon
Purpose:  Destructor. Destroys the message filter objects stored.
**********/

MessageFilterCon::~MessageFilterCon()
{
  MessageFilter *TempPointer;
  MessageFilter *DeletePointer; 
  HostList *TempPointer2;
  HostList *DeletePointer2; 
  PeerList *PeerDel;
  PeerList *PeerPtr;
 
  // Delete list of filters
  TempPointer = Head;
  while (TempPointer != NULL){
    DeletePointer = TempPointer;
    TempPointer = TempPointer->Next;

    // Delete the accept list in the message filter.
    TempPointer2 = DeletePointer->FromHostList;
    while(TempPointer2 != NULL){
      DeletePointer2 = TempPointer2;
      TempPointer2 = TempPointer2->Next;
      delete DeletePointer2;
    }
    // Delete the forward list in the filter
    TempPointer2 = DeletePointer->ForwardHostList;
    while(TempPointer2 != NULL){
      DeletePointer2 = TempPointer2;
      TempPointer2 = TempPointer2->Next;
      delete DeletePointer2;
    }
    // Delete the peer list in the filter
    PeerPtr = DeletePointer->ForwardPeerList;
    while(PeerPtr != NULL){
      PeerDel = PeerPtr;
      PeerPtr = PeerPtr->Next;
      delete PeerDel;
    }

    // finally delete the filter
    delete DeletePointer;
  }

}


/**********
Function: MessageFilterCon::AddFilter
Purpose:  Adds a message filter object to the container.
          Returns 1 all the time.
**********/

int MessageFilterCon::AddFilter(MessageFilter *NewFilter)
{
  // If the list is empty put it at the top
  if(Head == NULL){
    Head = NewFilter;
  }
  // Otherwise put it at the end
  else{
    Last->Next = NewFilter;
  }   
  // Reset the last pointers
  Last = NewFilter;
  Last->Next = NULL;
  return 1; 
}


/**********
Function: MessageFilterCon::MatchExp
Purpose:  This function returns a threaded list (through the NewNext pointers)
          of message filters that match the message description.
**********/

MessageFilter *MessageFilterCon::MatchExp(char *Message)
{
  MessageFilter *ReturnFilters = NULL;
  MessageFilter *ReturnFiltersLast = NULL;
  MessageFilter *TempPointer;

  TempPointer = Head;

  // For every filter in the list do
  while(TempPointer != NULL){
 
    // Run the regexp on the message
    if(Match(Message, TempPointer->RegExp)){
      // Create a thread in the list of filters to the matched filters list
      if(ReturnFilters == NULL){
	ReturnFilters = TempPointer;
	ReturnFilters->NewNext = NULL; 
	// NewNext is used to thread the linked list with a new list.
	// We want to do this so we don't mess up the original list, but
	// we use the same memory. 
	ReturnFiltersLast = ReturnFilters;

      }
      // If it doesn't match don't include this filter in the thread
      else{
	ReturnFiltersLast->NewNext = TempPointer;
	ReturnFiltersLast = TempPointer;
	ReturnFiltersLast->NewNext = NULL; 
      }
    }

    TempPointer = TempPointer->Next;
  }

  // Return the head of the list.
  return ReturnFilters;
}


/**********
Function: MessageFilterCon::Match
Purpose:  This function calls the system's regular expression functions
          to match the message description to the message filter
	  regular expression.
**********/

int MessageFilterCon::Match(char *String, char *Pattern)
{

  int i;
  regex_t re;
  
  // Compile the regular expression
  i=regcomp(&re, Pattern, REG_EXTENDED|REG_NOSUB);
  if (i != 0) return(0);
  // Run the regular expression on the input
  i = regexec(&re, String, (size_t) 0, NULL, 0);
  regfree(&re);
  if (i != 0) return(0);
  return(1);
}
