/****************************************************************
Server.C
CS 481 - Team A
Spring 1997
Hummingbird Project - Dr. Frincke
Coded by: Pete Neisen
Description: This file contains the implementation of the Server object.
*****************************************************************/
/*
 Copyright (C) 1997-1998 The HummingBird Project - hummer@cs.uidaho.edu

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

static char Server_C[] = "$Id: Server.C,v 1.14 1998/03/09 04:37:21 marc9442 Exp $";

#include "Server.h"            // Server class definition.

/**********
Function: Server::Server
Purpose:  Basic constructor.
**********/

Server::Server()
{
  ServerMessageHandler = NULL; // Init message handler variable.
}


/**********
Function: Server::Server(int NewPort)
Purpose:  Constructor that accepts the port number.
**********/

Server::Server(int NewPort)
{
  // Set port, init message handler
  Port = NewPort;
  ServerMessageHandler = NULL;
}


/**********
Function: Server::~Server
Purpose:  Basic destructor.
**********/

Server::~Server()
{
}


/**********
Function: Server::StartServer
Purpose:  This function sets up the server socket and contains the main loop
          of the server. This function is where the server spends most of its
	  time.
**********/

void Server::StartServer(void)
{
  // Create the sockets and initialize.
  InSocket = new sock_serv();
  InSocket->create();

  // Bind the socket to the message port  
  if (!InSocket->bind(Port, MESSAGE_SERVICE)) {
    Print_Error("Server", "Socket Bind Error. Is there a hummer already running on this machine?");
    exit(1);
    }

    // Write the PID to file.
  PgDatabase *piddb = OpenDB();
  
  QueryDB(&piddb,"delete from pid where name = 'Server';");

  char *query;
  query = new char[NORMAL_STRING_SIZE];
  sprintf(query, "insert into pid values ('Server', %ld);", getpid()); 
  
  QueryDB(&piddb,query);
  
  delete [] query;
  delete piddb;


  // Listen for connections
  InSocket->listen(SERVER_LISTEN_BACKLOG);

  // Set Reload and ServerRun to initial values.
  ReloadHandler = 1;
  ServerRun = 1;

  // Do until exit request
  while(ServerRun) {       

    // Wait for connection to listening socket
    if (InSocket->accept()) {

      // Check to see if the configuration has changed.
      // If it has, reload the handler.  *Must be done in parent*
      if(ReloadHandler) LoadHandler();

      // Fork child to handle request
      if (fork() == 0) {
	// Pass off connection to sock_cli object
	ConnectedSocket = InSocket->make_sock_cli();
	// Authenicate the client
	if (!ConnectedSocket->recv_auth(MESSAGE_KEYTAB)) {
	  ConnectedSocket->close();
	  delete ConnectedSocket;
	  exit(0);
	}
	
	// Don't handle the signals as we do in the parent
	signal(SIGHUP, SIG_IGN); 
	signal(SIGINT, SIG_IGN);

	// Handle the incomming message
	ProcessRequest();      

	// Close the listening socket in the child, not needed here
	InSocket->close();
	delete InSocket;
	
	// close out the connection and exit
	ConnectedSocket->close();
	delete ConnectedSocket;

	exit(0);
      }

      else {
	// Parent
	// Close the accepted socket in the parent, not needed here
	InSocket->close_a();
      }
    }
  }

  // Server was killed. Shutdown gracefully.
  
  // Remove pid from db
  piddb = OpenDB();
  QueryDB(&piddb,"delete from pid where name = 'Server';");
  delete piddb;

  // Close listening socket.
  InSocket->close();
  delete InSocket;
  delete ConnectedSocket;
  delete ServerMessageHandler;
}


/**********
Function: Server::ProcessRequest
Purpose:  This function reads in the incoming message, parses it into a 
          message object and sends it to the message handler. It also
	  checks the message age to prevent forever looping messages.
**********/

void Server::ProcessRequest(void)
{
  char *InMessage;
  char *PeerAddr;
  Message *TempMessage;

  // Allocate memory
  TempMessage = new Message;
  InMessage = new char[BUFFER_SIZE];
  PeerAddr = new char[IP_ADDRESS_SIZE];

  //Init the buffer.
  bzero(InMessage, BUFFER_SIZE);

  // Read in the message from the socket.
  int ib;
  ib = ConnectedSocket->read(InMessage, BUFFER_SIZE);
  InMessage[ib] = '\0';
  
  // Parse the incoming string into a message object.
  TempMessage->MessageAge = atoi(strtok(InMessage,"|"))+1;
  strcpy(TempMessage->Date, strtok(NULL,"|"));
  strcpy(TempMessage->Time, strtok(NULL,"|"));
  strcpy(TempMessage->Description, strtok(NULL,"|"));
  strcpy(TempMessage->HostIP, strtok(NULL,"|"));
  strcpy(TempMessage->Filename, strtok(NULL,"|"));
  strcpy(TempMessage->Tool, strtok(NULL, "|"));

  // Check the message age. Kill it if it is too old.
  if(TempMessage->MessageAge < MAX_MESSAGE_AGE){ 
    
    // Send the message to the handler.
    InSocket->get_rmt_addr(PeerAddr);
    ServerMessageHandler->HandleMessage(TempMessage, PeerAddr);

  } // else the message will die. 

  // Free memory.
  delete TempMessage;
  delete[] InMessage;
  delete[] PeerAddr;
}


/**********
Function: Server::KillServer
Purpose:  This function sets ServerRun to 0 so the loop will exit on the
          next message received.
**********/

void Server::KillServer(void)
{
  ServerRun = 0;
}


/**********
Function: Server::LoadHandler
Purpose:  This function deletes the old message handler if it exists and 
          creates a new one with the updated config files.
**********/

void Server::LoadHandler(void)
{
  // Delete the old object
  if(ServerMessageHandler) delete ServerMessageHandler;

  // Create a new one and load the configuration 
  ServerMessageHandler = new MessageHandler;
  ServerMessageHandler->LoadConfigFiles();  
  ReloadHandler = 0;
}


/**********
Function: Server::SetReloadHandler
Purpose:  This function sets the ReloadHAndler flag. This causes the message
          handler to be destroyed and recreated with the new changes in the
	  configuration files.
**********/

void Server::SetReloadHandler(void)
{
  ReloadHandler = 1;
}

