/****************************************************************
ToolIMain.C
CS 481 - Team A
Spring 1997
Hummingbird Project - Dr. Frincke
Coded by: Pete Neisen
Description: This file contains the tool interface. The tool interface is 
             the program that the log tools, alert tools, and the hummer
	     itself uses to pass messages into the hummingbird system.
*****************************************************************/
/*
 Copyright (C) 1997-1998 The HummingBird Project - hummer@cs.uidaho.edu

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

static char ToolIMain_C[] = "$Id: ToolIMain.C,v 1.14 1998/03/06 23:00:15 marc9442 Exp $";

#include "Hummer.h"         // Global hummer defines.
#include "sock_cli.h"
#include "Message.h"        // The message object.
#include "Print_Error.h"    // Print_Error fuction.
#include <iostream.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <ctype.h>

// Function prototypes
void PrintInstruct(void);
int ParseParameters(int ArgCount, char **Args, Message *ParamData);
int ValidateParameters(Message *ParamData);
int CheckDateTime(char *Date);
int SendToHummer(Message *ParamData);


/**********
Function: Main
Purpose:  This is the main function of the tool interface. It handles
          the input passed into it via command line parameters. 
**********/

int main(int argc, char **argv)
{
  Message ParamData;

  // Init our structure
  ParamData.Description[0] = '\0';
  ParamData.Date[0] = '\0';
  ParamData.Time[0] = '\0';
  ParamData.Filename[0] = '\0';
  ParamData.HostIP[0] = '\0';
  ParamData.Tool[0] = '\0';

  if(argc != 1){
    // Parse the command line.
    if(ParseParameters(argc, argv, &ParamData)){
      PrintInstruct();
      return 0;
    }
    // Make sure the parameters passed in are semi valid.
    else if(ValidateParameters(&ParamData)){
      PrintInstruct();
      return 0;
    }

    // Send the info to the hummer.
    SendToHummer(&ParamData);
  }
  else{
    PrintInstruct();        // Print the instructions 
    return 0;
  }
}


/**********
Function: PrintInstruct
Purpose:  Print the instructions on how to call ToolI to stdout.
**********/

void PrintInstruct(void)
{
  // Dump the instructions
  cerr << "usage: ToolI -m <message text> -d <date> -t <time> -h <hostname>"
       << endl << "             [-f <file>] [-s <tool>]" << endl;
}


/**********
Function: ParseParameters
Purpose:  Parse the command line parameters and place the the data in the
          message object. This function also report errors in the command
	  line so the instructions can be printed.
**********/

int ParseParameters(int ArgCount, char **Args, Message *ParamData)
{
  int c;
  
  // Use getopt to parse out command line options
  while ((c = getopt (ArgCount, Args, "m:d:t:h:f:s:")) != -1) {
    switch (c)
      {
	// Copy the message over
      case 'm':
	strncpy(ParamData->Description, optarg, NORMAL_STRING_SIZE-1);
	ParamData->Description[NORMAL_STRING_SIZE-1] = '\0';
	break;
	// Copy the date over
      case 'd':
	strncpy(ParamData->Date, optarg, DATE_SIZE-1);
	ParamData->Date[DATE_SIZE-1] = '\0';
	break;	
	// Copy the time over
      case 't':
	strncpy(ParamData->Time, optarg, TIME_SIZE-1);
	ParamData->Time[TIME_SIZE-1] = '\0';
	break;	
	// Copy the host address over
      case 'h':
	strncpy(ParamData->HostIP, optarg, IP_ADDRESS_SIZE-1);
	ParamData->HostIP[IP_ADDRESS_SIZE-1] = '\0';
	break;
	// Copy the filename over
      case 'f':
	strncpy(ParamData->Filename, optarg, NORMAL_STRING_SIZE-1);
	ParamData->Filename[NORMAL_STRING_SIZE-1] = '\0';
	break;
	// Copy the tool name over
      case 's':
	strncpy(ParamData->Tool, optarg, NORMAL_STRING_SIZE-1);
	ParamData->Tool[NORMAL_STRING_SIZE-1] = '\0';
	break;
	// If it was something else print out an error
      case '?':
	if (isprint (optopt))
	  cerr << "Unknown option `-" << (char) optopt << "'." << endl;
	else
	  cerr << "Unknown option character `\\x" << hex << optopt << "'." << endl;
	return 1;
      default:
	return 1;
      }
  }
  
  return 0;
}

/**********
Function: ValidateParameters
Purpose:  Make sure the command line arguments are within some requirements
**********/

int ValidateParameters(Message *ParamData)
{
  // Check to make sure the required fields were specified.
  if(!ParamData->Description[0] || !ParamData->Date[0] ||
     !ParamData->Time[0] || !ParamData->HostIP[0]) { 
    cerr << "Required Fields not filled" << endl;
    return 1;
  }

  // Check the format of the date and time parameters.
  else if(CheckDateTime(ParamData->Date) || CheckDateTime(ParamData->Time)) {
    cerr << "Date or Time not correct format" << endl;
    return 1;
  }
  else return 0;
}


/**********
Function: CheckDateTime
Purpose:  This function checks to make sure the date and time are valid. 
          At this point this just checks to make sure they are not zero.
**********/

int CheckDateTime(char *Date)
{
  int TempDate;

  // The date/time must form an int
  TempDate = atoi(Date);
  if(TempDate<1) return 1;
  else return 0;
}


/**********
Function: SendToHummer
Purpose:  Opens a socket to the message server and send the message over. 
**********/

int SendToHummer(Message *ParamData)
{
  // init memory, and set up a outgoing socket
  char *OutMessage;
  sock_cli *HummerSocket;
  
  HummerSocket = new sock_cli();
  HummerSocket->create();
  
  // Get the local host name
  char *hostname;
  hostname=new char[MAXHOSTNAMELEN];
  
  if (gethostname(hostname, MAXHOSTNAMELEN) < 0) {
    perror("gethostname");
    exit(1);
  }

  // Connect to the local host
  if(HummerSocket->connect(hostname, MESSAGE_PORT, MESSAGE_SERVICE)) {
    OutMessage = new char[BUFFER_SIZE];

    // Set up the outgoing message.
    sprintf(OutMessage, "%d|%s|%s|%s|%s|%s|%s",
	    0,
	    ParamData->Date,
	    ParamData->Time,
	    ParamData->Description,
	    ParamData->HostIP,
	    ParamData->Filename[0] ? ParamData->Filename : "0",
	    ParamData->Tool[0] ? ParamData->Tool : "0");

#ifdef DEBUG_TOOLI
    Print_Error("ToolI", "OutMessage: %s", OutMessage);
#endif

    // Authenticate ourself to the server
    if (!HummerSocket->send_auth()) {
      Print_Error("ToolI", "Send_Auth Failure");
      HummerSocket->close();
      delete HummerSocket;
      return 1;
    }
    // Write out the message
    HummerSocket->write(OutMessage);

    // Close out the connect 
    HummerSocket->close();
    delete[] OutMessage;

    delete HummerSocket;
    return 0;
  }
  // We couldn't connect print an error.
  else{
    Print_Error("ToolI", "Failed to connect to %s", hostname);
    HummerSocket->close();
    delete HummerSocket;
    return 1;
  }
  
}
