#! &PERL& -I&INCDIR&
# -*- perl -*-
#must include the line above to use perl v5.002 on this system.

 
 ################################################################
 #      Hummingbird  (Team A)
 #      07/05/97
 #      Dean Polla
 #
 #	This tool is designed to parse the hummer log for  
 #	a regular expression  and send an alert message	
 #	to the hummer if it is found.  The regular expressiong is
 # 	passed in thru the command line option -string <exp>
 #
 ################################################################

#
# Copyright (C) 1997-1998 The HummingBird Project - hummer@cs.uidaho.edu
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

forktoBg($0);

use Toollib;
use Getopt::Long;


#################################################################
# Command Line Options
#################################################################


$result = GetOptions('string=s',	\$pattern,
		     'timeframe:i',	\$time_frame,
		     'threshold:i',	\$threshold,
		     'w',		\$w_option,
		     's:i',		\$sleepTime);


if ($result == 0)	{ print "Command Line Error \n"; }
if ($time_frame eq "")	{ $time_frame = 30; }
if ($threshold  eq "")	{ $threshold  = 1;  }
if ($sleepTime eq "")  { $sleepTime = 5;  }


$logfile = join('', "./", $host, "/", "hummer.log");

while (!$do_exit) { &parseData(); }

&delpid();

#################################################################
# Parse Data
#################################################################

sub parseData {

open LOG,$logfile || die "Can't open hummer log.\n";

seek LOG,0,2;
seek LOG,0,1;

$first_pass = 1;
$hits = 0;

while (!$do_exit) {
 while(<LOG>) {
  chop;
 
   if ((/$pattern/) && (!/ALERT/)){
      ($trust,$date,$time,$message,$host,$slog) = split(/\|/,$_);
    
	$hh = substr($time,0,2);		# get hours and minutes
        $mm = substr($time,2,2);		# from the time stamp

      if ($first_pass) {
	$base_minutes = $hh*60+$mm;
	$current_minutes = $base_minutes;	
      	$first_pass = 0;
	}
	else {					# convert time into 
	  $current_minutes = $hh*60+$mm;	# minutes
	}

	$elapsed_time = $current_minutes - $base_minutes;
	
	if ($elapsed_time <= $time_frame) {	
		$hits += 1;
	}	
	
	if ($elapsed_time > $time_frame) { 
		$base_minutes = $current_minutes;
		$hits = 1;
		$elapsed_time = 0;
	}

	if ($hits >= $threshold) {
		&sendAlert();
	}


	# if -w write data
	if ($w_option) { &writeAlertData(); }
      } # outer if 
   } # inner while 

 $SIG{USR1} = \&signal_handler;      	# Set signal handler to 
				     	# catch USR1 interrupt and
   if ($reinitialize == 1) {		# reinitialize the filehandle
	$reinitialize = 0;		# if the signal is sent.
	last;
   }
   else {
	sleep $sleepTime;
	seek LOG,0,1; 
   }
 }  # outer while (1)

close(LOG);
}



##################################################################
# This subroutine will write the data to the Alert Log File.
##################################################################
sub writeAlertData {
 open (ALOG, ">>./$pattern.alog") or die "Error - couldn't open $pattern.log.\n";
     print ALOG "$date   |  $time   |  $message \n";
 close (ALOG);
}



##################################################################
# This subroutine will send an alert to the hummer.
##################################################################
sub sendAlert {
    
    $msngr = join('', "ALERT  ",$pattern,"  hits - (",$hits,")  minutes - (",
		  $elapsed_time,")  ",$message);
    
    ToolI($msngr, $date, $time, $hostname, $logfile, "alert");
} 



#############################################################
# Signal handler kills background process
#############################################################
sub signal_handler {
  local($sig) = @_;
   print "Received a SIG$sig--reinitializing file:  $0\n";
   $reinitialize = 1;
} 
