// ***************************************************************************
// DefaultWindow Class
// 
//
// Class Description:  
// ------------------
// Purpose: This class is a subwindow for the system default settings.  The system
// will place two default settings at start up.  This class provides an interface 
// for the user to change those setttings.  This interface is a dialog box with 
// two TextFields and a submit button.   When a user activates the Set Defaults 
// option under the Options menu, this  class becomes instantiated and will 
// popup a dialog box for the user.  This class instantiates and lays out the 
// GUI components for this interface/dialog.  In addition this class implements 
// its own event handling and when instantiated this class takes control of the 
// focus of the Atom window.
//
// System Relationship:  Intercommunication with the whole system is non-existence.  
//
// Requirements:  This class requires the Java Developemnt Kit (JDK) 1.0.
// Aside from this there are no special algorithms, or constraints.  
//
// Pre-Conditions:  Atom.class.
//
// Post Conditions: False.
//
// Error Handling:  Addressed with the try and catch clause.
//
//
// Class Specification:
// --------------------
// This class is a subsequent window of the main interface, it is an option dialog.
// It is a dialog box that requests and receives focus when it is instantiated.  It 
// prompts the user for the following:  Tool Class, and  Directory.  Each item is
// a TextField for the user to enter the appropriate information into.  The Tool
// Class TextField defaults to the system Tool Class default as does the Directory. 
// The directory is based off of the present working directory.
//
//
// Constructor
// -----------
//      public DefaultWindow (Frame parent, String title) 
//
// Event Handling 
// --------------
//      public boolean action(Event evt, Object arg)
//      public boolean gotFocus(Event event, Object arg)
//
// *****************************************************************************

import java.awt.*;
import java.lang.*;
import java.awt.*;
import java.awt.event.*;

public class DefaultWindow extends Dialog implements ActionListener, FocusListener {

  Panel      Panel              = new Panel();                     // Sub-container for the components
  TextField  toolClassTextField = new TextField(Atom.toolClass);   // TextField for the Tool Class
  TextField  directoryTextField = new TextField(Atom.directory);   // TextField for the Directory

  Button     submitButton;                                         // Submit Button

  Constrain con = new Constrain();

  // *****************************************************************************
  // public DefaultWindow (Frame parent, String title)
  //
  // Constructor Description:  The constructor for the DefaultWindow class.  Initializes
  // new variables and components, and lays out the components on the DefaultWindow dialog
  // interface.
  //
  // Constructor Specification:  The constructor lays out the components for 
  // each area on the DefaultWindow interface.  It uses the GridBagLayout Manager.  The 
  // following is a listing of the interface layout and its corresponding components:
  // 
  //   Position  Description               Components
  //   --------  -----------               ----------
  //    * Frame  DefaultWindow Interface:  GridBagLayout Manager
  //    * Center Tool Class TextField      toolClassTextField
  //             Directory TextField       directoryTextField
  //           
  // The constructor will be accessed each time the Set Defaults is activated.
  // 
  // Pre-Conditions:  Set Defaults under options on the menu bar is activated.
  //
  // Post-Conditions: Information is stored in the Atom.class variables.
  //
  // *****************************************************************************

  public DefaultWindow (Frame parent, String title) {
    super(parent, title, true);

    this.setLayout(new GridBagLayout());
    Panel.setLayout(new GridBagLayout());
 
    // *** Initialize Button ****
    submitButton= new Button("Submit");
    submitButton.setFont(new Font("Dialog", Font.BOLD, 12));
    submitButton.setForeground(Color.blue);

    // LISTENERS
    submitButton.addActionListener(this);

    // The constrain method places the components onto the interface using the
    // GridBagLayout manager.                
    con.constrain(Panel, new Label("Tool Class"), 0,0,1,1, 0,20,0,0);
    con.constrain(Panel, toolClassTextField, 0,1,1,1, 0,20,0,0); 
    con.constrain(Panel, new Label("Directory"), 0,2,1,1, 10,20,0,0); 
    con.constrain(Panel, directoryTextField, 0,3,1,1, 0,20,0,0);    
   
    con.constrain(Panel, submitButton, 0,4,1,1, GridBagConstraints.NONE,
                            GridBagConstraints.CENTER, 1.0, 1.0, 10, 0, 5, 5);

    con.constrain(this, Panel, 0,1,1,1, GridBagConstraints.NONE,
                          GridBagConstraints.NORTH, 1.0, 1.0, 15, 5, 5, 5);
    this.pack();

  } // constructor




  // ***************************************************************************** 
  //  public void actionPerformed(ActionEvent evt)
  //
  // Method Description: This method handles all of the events of type ActionEvent
  // generated by the components in the DefaultWindow interface.
  //
  // Method Specification:  If an ActionEvent occurs this method will handle it.
  // Events are generated by the user activating a component.  Such as, clicking a
  // button, or selecting a list item.  The following lists the events and reactions
  // for each ActionEvent:  
  //
  //        Label       Component/Event     Reaction
  //        -----       ---------------     --------
  //      * Submit      submitButton        All the TextField information is collected and
  //                                        stored in the Atom.class static variables.    
  //
  // This method will be accessed very infrequently.
  //
  // Pre-Condition:  Component objects have been instantiated.
  // Post-Condition: Information has been collected from the TextFields and stored in the 
  // Atom.class static variables.
  //
  // *****************************************************************************

  public void actionPerformed(ActionEvent evt) {
      Object source = evt.getSource();
      String toolClassInfo = "";
      String directoryInfo   = "";
    
      // -----------------
      // Submit Button
      // -----------------

      // Collect information from the TextFields.
      // Collection Format: <Tool Class> <Directory> <Default>
      if (source  == submitButton) {
	  toolClassInfo   = toolClassTextField.getText();
	  directoryInfo   = directoryTextField.getText();

	  // Store information in the static variables of the Atom.class
	  Atom.directory = directoryInfo;
	  Atom.toolClass = toolClassInfo;
	  
	  this.setVisible(false);
	  this.dispose();
      } //submit button
  } // actionPerformed


  // Focus methods
  public void focusGained(FocusEvent evt) { }
  public void focusLost(FocusEvent evt) { }


} // AddPopup













