##############################################################
#      Hummingbird  (Team A)
#      12/02/97
#      Jamie Marconi
#      
# Module Description
#
# This CGI creates HTML to control HBVT and is the action CGI for the forms 
# and image maps contained within that HTML. This is the module of HBVT that
# controls the user interface.
#
# Module Specification
#
# This module along with supporting html files fill the requirements of
# providing the user will access to the program.  It will handle the users
# selection of the image for more detail.  It will also allow the user to 
# select different dates log data to plot.
#
# Internal Data Description
#
# This module makes uses of CGI a Simple Common Gateway Interface Class for
# Perl5.  This Perl class makes it ease to create fill out forms and parse
# their contents.  Each part of a form in HTML is given a logical name address,
# with CGI these names are transformed into Perl string variables when the 
# class is created.  
#
##############################################################


#
# Copyright (C) 1997-1998 The HummingBird Project - hummer@cs.uidaho.edu
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

package Formlib;

require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw($host goform);

use Sys::Hostname;
$host = hostname();

# Function called by a form to start the generic form interface.
#
# TITLE is the title of the HTML page
# FORMCGINAME is the self referancing name for the image map
# GRAPHCGINAME is the name of the graph cgi associated with this form.
# TOOL is the name of the HB tool that is queried on.
# ADDSQL is any additional SQL that may be needed for the table view.

sub goform {

local($TITLE, $FORMCGINAME, $GRAPHCGINAME, $TOOL, $ADDSQL) = @_;

# Start the HTML output
use CGI qw(:standard);

$query = new CGI;

print $query->header(-expires=>'now');

# Get the date for the form.
if (!$query->param('date')) {
    $date = `date \'+%m/%d/%Y\'`;
    chop $date;
}
else {
    $date = $query->param('date');
}

# Check to see that its in the correct format.
if (!($date =~ /\d\d\/\d\d\/\d\d\d\d/)) {
    print "Date not correct format\n";
    exit;
}

# If hour is on the CGI query then this is a table view
if ($query->param('hour')) {
    $hour = $query->param('hour');

    #Check to see that its in the correct format.
    if (!($hour =~ /-?\d+/)) {
	print "Hour not correct format\n";
	exit;
    }
    if ($hour eq -1) { $hour = 0; }
    $time_low = "$hour:00:00";
    $hour++;
    $time_high = "$hour:00:00";

    # Write out the HTML output from psql
    open(PSQL, "/usr/local/pgsql/bin/psql -c \"select * from log where 
tool='$TOOL' and ldate='$date' and ltime >= '$time_low' and ltime <= 
'$time_high' $ADDSQL;\" -d hummer_$host -H|");

    while (<PSQL>) {
	print $_;
    }
    exit;
}

# Write out the HTML for the form.
print $query->start_html(-title=>$TITLE,
			 -author=>'marc9442@cs.uidaho.edu',
			 -TEXT=>'black', 
			 -BGCOLOR=>'white', 
			 -LINK=>'blue', 
			 -VLINK=>'purple');

print $query->h1($TITLE),
      "<center>\n";

print $query->start_form(-method=>'GET');

print <<EOF;
<table border><tr><td>
A specific day (MM/DD/YYYY)<BR>
<input type=text size=10 maxlength=10 value=\"$date\" name=date>
<input type=submit value=\"Graph!\">
</td></tr></table>
EOF

print $query->end_form,
      "</center>\n",
      $query->hr;

# Print out the Image Map HTML
print_imgmap($FORMCGINAME, $date);

# Print out the IMG tag for the graph module.
print "<center><img border=0 src=\"$GRAPHCGINAME?date=$date\" height=300 width=604 alt=\"Graph\" usemap=\"\#graph_map\"></center>\n", $query->hr;

print $query->end_html;
}

# Function called by main to print Image Map
#
# $cgi is the self referancing name
# $date is the date for this form instance.

sub print_imgmap {

local($cgi, $date) = @_;
    
print <<EOF;
<map name="graph_map">
<area shape="rect" coords="26,40,49,242" href="$cgi?date=$date&hour=-1">
<area shape="rect" coords="50,40,73,242" href="$cgi?date=$date&hour=1">
<area shape="rect" coords="74,40,97,242" href="$cgi?date=$date&hour=2">
<area shape="rect" coords="98,40,121,242" href="$cgi?date=$date&hour=3">
<area shape="rect" coords="122,40,145,242" href="$cgi?date=$date&hour=4">
<area shape="rect" coords="146,40,169,242" href="$cgi?date=$date&hour=5">
<area shape="rect" coords="170,40,193,242" href="$cgi?date=$date&hour=6">
<area shape="rect" coords="194,40,217,242" href="$cgi?date=$date&hour=7">
<area shape="rect" coords="218,40,241,242" href="$cgi?date=$date&hour=8">
<area shape="rect" coords="242,40,265,242" href="$cgi?date=$date&hour=9">
<area shape="rect" coords="266,40,289,242" href="$cgi?date=$date&hour=10">
<area shape="rect" coords="290,40,313,242" href="$cgi?date=$date&hour=11">
<area shape="rect" coords="314,40,337,242" href="$cgi?date=$date&hour=12">
<area shape="rect" coords="338,40,361,242" href="$cgi?date=$date&hour=13">
<area shape="rect" coords="362,40,385,242" href="$cgi?date=$date&hour=14">
<area shape="rect" coords="386,40,409,242" href="$cgi?date=$date&hour=15">
<area shape="rect" coords="410,40,433,242" href="$cgi?date=$date&hour=16">
<area shape="rect" coords="434,40,457,242" href="$cgi?date=$date&hour=17">
<area shape="rect" coords="458,40,481,242" href="$cgi?date=$date&hour=18">
<area shape="rect" coords="482,40,505,242" href="$cgi?date=$date&hour=19">
<area shape="rect" coords="506,40,529,242" href="$cgi?date=$date&hour=20">
<area shape="rect" coords="530,40,553,242" href="$cgi?date=$date&hour=21">
<area shape="rect" coords="554,40,577,242" href="$cgi?date=$date&hour=22">
<area shape="rect" coords="578,40,602,242" href="$cgi?date=$date&hour=23">

<area shape="rect" coords="14,243,37,289" href="$cgi?date=$date&hour=-1">
<area shape="rect" coords="38,243,61,289" href="$cgi?date=$date&hour=1">
<area shape="rect" coords="62,243,85,289" href="$cgi?date=$date&hour=2">
<area shape="rect" coords="86,243,109,289" href="$cgi?date=$date&hour=3">
<area shape="rect" coords="110,243,133,289" href="$cgi?date=$date&hour=4">
<area shape="rect" coords="134,243,157,289" href="$cgi?date=$date&hour=5">
<area shape="rect" coords="158,243,181,289" href="$cgi?date=$date&hour=6">
<area shape="rect" coords="182,243,205,289" href="$cgi?date=$date&hour=7">
<area shape="rect" coords="206,243,229,289" href="$cgi?date=$date&hour=8">
<area shape="rect" coords="230,243,253,289" href="$cgi?date=$date&hour=9">
<area shape="rect" coords="254,243,277,289" href="$cgi?date=$date&hour=10">
<area shape="rect" coords="278,243,301,289" href="$cgi?date=$date&hour=11">
<area shape="rect" coords="302,243,325,289" href="$cgi?date=$date&hour=12">
<area shape="rect" coords="326,243,349,289" href="$cgi?date=$date&hour=13">
<area shape="rect" coords="350,243,373,289" href="$cgi?date=$date&hour=14">
<area shape="rect" coords="374,243,397,289" href="$cgi?date=$date&hour=15">
<area shape="rect" coords="398,243,421,289" href="$cgi?date=$date&hour=16">
<area shape="rect" coords="422,243,445,289" href="$cgi?date=$date&hour=17">
<area shape="rect" coords="446,243,469,289" href="$cgi?date=$date&hour=18">
<area shape="rect" coords="470,243,493,289" href="$cgi?date=$date&hour=19">
<area shape="rect" coords="494,243,517,289" href="$cgi?date=$date&hour=20">
<area shape="rect" coords="518,243,541,289" href="$cgi?date=$date&hour=21">
<area shape="rect" coords="542,243,565,289" href="$cgi?date=$date&hour=22">
<area shape="rect" coords="566,243,589,289" href="$cgi?date=$date&hour=23">
</map>
EOF
}

1;
