##############################################################
#      Hummingbird  (Team A)
#      12/02/97
#      Jamie Marconi
#      
# Module Description
#
# This CGI creates a GIF image that contains the daily plot.
#
# Module Specification
#
# This module is to fill the requirements of transforming columns of normalized
# data into graphic data plot.  It will make sure larger normalized data is
# represented by taller bars in data plot.  If no data exists for specified
# date the plot will be blank.  Bar heigh will only be made up of data from
# specified date.
#
# Internal Data Description
#
# The graph module uses GD.pm Interface to Gd Graphics Library.  The
# Gd Graphics library is a library to create and manipulate GIF images.  It
# provides the ability to draw lines, boxes, and text on GIF images.  All other
# internal data is temporary strings.
#
##############################################################

#
# Copyright (C) 1997-1998 The HummingBird Project - hummer@cs.uidaho.edu
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

package Graphlib;

require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw($SPD $host $date title plot_data label_x_axis grid label_y_axis
	     legend print_img print_ht_img);

use Sys::Hostname;
use CGI;
use GD;

# Function to draw the Titles on the image
#
# $title1 is the top title
# $title2 is the lower title

sub title {
    local($title1, $title2) = @_;
    $im->string(gdLargeFont,
		$BIGGRAPHY/2 -(length($title1) * $gdLargeFont-width/2),
		0,$title1,$black);
    $im->string(gdLargeFont,
		$BIGGRAPHX/2 - (length($title2) * gdLargeFont->width/2),
		gdLargeFont->height+4,
		$title2,$black);
}

# Function to draw two bars of the plot
#
# $first is the height of the first bar
# $second is the height of the second bar
# $x*2 is the x position of first bar.

sub plot_data {
    local($first, $second, $x) = @_;

    $pt = (($first)*200);  # temp data
    $im->line($x*2+$GRAPHMARGIN, $bot, $x*2+$GRAPHMARGIN, $bot-$pt,
	      $green);
    $im->line($x*2+$GRAPHMARGIN, $bot-$pt-1, $x*2+$GRAPHMARGIN, $top,
	      $gray);
    
    $pt = (($second)*200);  # temp data
    $im->line($x*2+$GRAPHMARGIN+1, $bot, $x*2+$GRAPHMARGIN+1, $bot-$pt,
	      $blue);
    $im->line($x*2+$GRAPHMARGIN+1, $bot-$pt-1, $x*2+$GRAPHMARGIN+1, $top,
	      $gray);
}

# Function to draw the x axis labels
# 
# $format is a sprintf format string
# $high is the highest label value

sub label_x_axis {
    local($format, $high) = @_;
    
    local($xlabel) = 0.0;

    for ($i=0; $i<=10; $i++) {
	$string = (sprintf "$format", $xlabel);
	$im->string(gdSmallFont, $GRAPHMARGIN - (gdSmallFont->width*4),
		    $bot - (gdSmallFont->height/2) - $i*20,
		    $string, $black);
	$xlabel = $xlabel + ($high/10);
    }
}

# Function to draw the grid on the image

sub grid {
    for ($y=0; $y<11; $y++) {
	for ($x=0; $x<25; $x++) {
	    $im->setPixel($GRAPHMARGIN+$x*24,
			  $bot-$y*20, $black);
	}
    }
}

# Function to draw the times along the y axis as labels.

sub label_y_axis {
    for ($i=0; $i<24; $i++) {
	$string = sprintf "%2d:00", (($i%12) eq 0 ? 12 : $i%12);
	$string .= ( $i<12 ? "a" : "p") . "m\n";
	$im->stringUp(gdSmallFont, 
		      $GRAPHMARGIN - (gdSmallFont->height/2) + $i*24,
		      $bot + (gdSmallFont->width*7) + 4, 
		      $string, $black);
    }
}

# Function to draw the legend of colors at the bottom of the image
#
# $GREENLABEL is the label for the green color
# $BLUELABEL is the label for the blue color

sub legend {
    local($GREENLABEL, $BLUELABEL) = @_;

    $im->filledRectangle($GRAPHMARGIN, $BIGGRAPHY-8, $GRAPHMARGIN+8, 
			 $BIGGRAPHY, $green);
    $im->string(gdSmallFont, $GRAPHMARGIN+10, $BIGGRAPHY-10, $GREENLABEL, 
		$black);
    
    $im->filledRectangle($GRAPHMARGIN+140, $BIGGRAPHY-8, $GRAPHMARGIN+140+8,
			 $BIGGRAPHY, $blue);
    $im->string(gdSmallFont, $GRAPHMARGIN+140+10, $BIGGRAPHY-10,
		$BLUELABEL, $black);
}

# Fucntion to convert the image to GIF and print for the web server

sub print_ht_img {
    print $query->header(-type=>'image/gif',
			 -expires=>'now');
    
    print $im->gif;
}

# Function to convert the image to GIF and print it on standard output

sub print_img {
    print $im->gif;
}

# Set up the Library for functions.

$query = new CGI;
$date = $query->param('date');
$host = hostname();

$SPD = (60 / 5) * 24;
$GRAPHMARGIN = 26;
$BIGGRAPHX = 604;
$BIGGRAPHY = 300;

# create a new image
$im = new GD::Image($BIGGRAPHX,$BIGGRAPHY);
# allocate some colors
$white = $im->colorAllocate(255,255,255);
$black = $im->colorAllocate(0,0,0);
$green = $im->colorAllocate(153, 51, 102);
$blue = $im->colorAllocate(153,153,255);
$gray = $im->colorAllocate(192,192,192);

# make the background transparent and interlaced 
$im->transparent($white);
#$im->interlaced('true');

$top = gdLargeFont->height*2+8;
$bot = 200 + $top;

1;
